@@ -168,22 +168,37 @@ class Resource(RestfulBase):
     def _get_parameter_names(self, *kinds):
         """Retrieve some subset of the resource's parameters."""
         names = []
-        for name in self._wadl_resource.parameter_names(
+        for parameter in self._wadl_resource.parameters(
             self.JSON_MEDIA_TYPE):
-            if name.endswith('_collection_link'):
-                if self.FIND_COLLECTIONS in kinds:
-                    names.append(name[:-16])
-            elif (name.endswith('_link')
-                  and name not in ('self_link', 'resource_type_link')):
-                # lazr.restful_obj.self will work, but is never
-                # necessary. lazr.restful_obj.resource_type is also
-                # unneccessary, and won't work anyway because
-                # resource_type_link points to a WADL description,
-                # not a normal lazr.restful resource.
-                if self.FIND_ENTRIES in kinds:
-                    names.append(name[:-5])
-            elif self.FIND_ATTRIBUTES in kinds:
-                names.append(name)
+            name = parameter.name
+            link = parameter.link
+            if (name != 'self_link' and link is not None
+                and link.can_follow):
+                # This is a link to a resource with a WADL
+                # description.  Since this is a lazr.restful web
+                # service, we know it's either an entry or a
+                # collection, and that its name ends with '_link' or
+                # '_collection_link', respectively.
+                #
+                # self_link is a special case. 'obj.self' will always
+                # work, but it's not useful. 'obj.self_link' is
+                # useful, so we advertise the scalar value instead.
+                if name.endswith('_collection_link'):
+                    # It's a link to a collection.
+                    if self.FIND_COLLECTIONS in kinds:
+                        names.append(name[:-16])
+                else:
+                    # It's a link to an entry.
+                    if self.FIND_ENTRIES in kinds:
+                        names.append(name[:-5])
+            else:
+                # There are three possibilities. This is not a link at
+                # all, it's a link to a resource not described by
+                # WADL, or it's the 'self_link'. Either way,
+                # lazr.restfulclient should treat this parameter as a
+                # scalar attribute.
+                if self.FIND_ATTRIBUTES in kinds:
+                    names.append(name)
         return names
 
     def lp_has_parameter(self, param_name):
